c======================================================================
c  This source code is public domain, and may be freely distributed,
c  modified, and used for any purpose.  IT COMES WITH ABSOLUTELY NO
c  WARRANTY OF ANY KIND.
c======================================================================
c   smi_to_2d.f -- produce 2D coordinates for a SMILES
c======================================================================
      program smi_to_2d
      include 'dt_f_smiles.inc'
      include 'dt_f_depict.inc'
c----------------------------------------------------------------------
      integer    MAXSMI
      parameter (MAXSMI = 2000)
      integer    MAXTDT
      parameter (MAXTDT = 4000)
      integer    mol, dep, atoms, atom
      character*(MAXSMI) smi, ismi, name
      character*(MAXTDT) tdt
      integer    smi_in, tdt_out, leninfile, ilun, smilen, ismilen
      integer    tdtlen, i, namelen, lenoutfile, olun
      character*(100) infile, outfile
      logical    ok, is_outfile
      real       x, y, z
c----------------------------------------------------------------------
c  Declare functions used.
c----------------------------------------------------------------------
      integer dt_f_smilin, dt_f_alloc_depiction, dt_f_calcxy
      integer dt_f_cansmiles
      integer dt_f_stream, dt_f_next
      logical dt_f_dealloc, dt_f_getcoord
c----------------------------------------------------------------------
c  Assign file units.
c----------------------------------------------------------------------
      ilun = 1
      olun = 2
c----------------------------------------------------------------------
c  Open input SMILES file and (maybe) output TDT file.
c----------------------------------------------------------------------
10    continue
      print '(/A$)', 'Input SMILES file: '
      read (*, fmt='(Q,A)', err=10) leninfile, infile
      open (unit=ilun, file=infile(:leninfile), status='old',
     &      access='sequential', err=999)
15    continue
      print '(/A$)', 'Output TDT file <stdout>: '
      read (*, fmt='(Q,A)', err=15) lenoutfile, outfile
      if (lenoutfile .eq. 0) then
        is_outfile = .FALSE.
      else
        open (unit=olun, file=outfile(:lenoutfile), status='unknown',
     &        access='sequential', err=999)
        is_outfile = .TRUE.
      endif
c----------------------------------------------------------------------
c  Main loop
c----------------------------------------------------------------------
      smi_in = 0
      tdt_out = 0
20    continue
      read ( unit=ilun, fmt='(Q,A)', end=1000) smilen, smi
      smi_in = smi_in + 1
c----------------------------------------------------------------------
c  Interpret SMILES.
c----------------------------------------------------------------------
      mol = dt_f_smilin(smi(:smilen))
      if (mol .eq. NULL_OB) then
        print *, 'SMILES rejected...'
      else
c----------------------------------------------------------------------
         dep = dt_f_alloc_depiction(mol)
         ok = dt_f_calcxy(dep)
         if (dep .eq. NULL_OB) then
            print *, 'Can''t create depict object...'
         else
c----------------------------------------------------------------------
c  Extract name.
c----------------------------------------------------------------------
            namelen = 0
            i = index(smi(:smilen), ' ')
            if (i .ne. 0) then
               name = smi(i+1:smilen)
               namelen = smilen - i
            endif
c----------------------------------------------------------------------
c  Put unique SMILES first in TDT.  Note that smilen is modified.
c----------------------------------------------------------------------
            smilen = dt_f_cansmiles(mol, .FALSE., smi)
            tdt = '$SMI<' // smi(:smilen) // '>'
            tdtlen = 5 + smilen + 1
c----------------------------------------------------------------------
c  Add name if present.
c----------------------------------------------------------------------
            if (namelen .gt. 0) then
               tdt(tdtlen+1:) = 'PCN<' // name(:namelen) // '>'
               tdtlen = tdtlen + 4 + namelen + 1
            endif
c----------------------------------------------------------------------
c  Add ISM datatype if isomeric.
c----------------------------------------------------------------------
            ismilen = dt_f_cansmiles(mol, .TRUE., ismi)
            if (ismi(:ismilen) .ne. smi(:smilen)) then
               tdt(tdtlen+1:) = 'ISM<' // ismi(:ismilen) // '>'
               tdtlen = tdtlen + 4 + ismilen + 1
            endif
c----------------------------------------------------------------------
c  Add 2D datatype.
c----------------------------------------------------------------------
            tdt(tdtlen+1:) = '2D<'
            tdtlen = tdtlen + 3
            atoms = dt_f_stream(mol, TYP_ATOM)
70          continue
            atom = dt_f_next(atoms)
            if (atom .eq. NULL_OB) go to 80
            ok = dt_f_getcoord(dep, atom, x, y, z)
            write ( tdt(tdtlen+1:tdtlen+7), '(1F7.2)' ) x
            tdt(tdtlen+8:tdtlen+8) = ','
            tdtlen = tdtlen + 8
            write ( tdt(tdtlen+1:tdtlen+7), '(1F7.2)' ) y
            tdt(tdtlen+8:tdtlen+8) = ','
            tdtlen = tdtlen + 8
            go to 70
80          continue
c----------------------------------------------------------------------
c  Remember to strip off last comma (just decrement tdtlen)...
c----------------------------------------------------------------------
            tdtlen = tdtlen - 1
            tdt(tdtlen+1:tdtlen+2) = '>|'
            tdtlen = tdtlen + 2
            call du_no_blanks(tdt(:tdtlen), tdtlen)
            if (is_outfile) then
              write (olun, '(a)') tdt(:tdtlen)
            else
              print '(a)', tdt(:tdtlen)
            endif
            tdt_out = tdt_out + 1
         end if
      end if
      ok = dt_f_dealloc(mol)
100   go to 20
c----------------------------------------------------------------------
c  Drop here to exit program
c----------------------------------------------------------------------
999   print *, 'ERROR (SMI_TO_2D) Can''t open input or output file...'
1000  continue
      print *, 'SMILES in: ', smi_in
      print *, 'TDTs out: ', tdt_out
      end
c======================================================================
c   du_no_blanks - Removes all blanks from str(:lenstr), and returns
c                  new lenstr.  str(newlength+1:oldlength) padded with
c                  blanks.
c======================================================================
      subroutine du_no_blanks(str, lenstr)
      character*(*) str
      integer lenstr, i, j
c----------------------------------------------------------------------
c  We remove blanks from left to right, from i = 1 to i = lenstr, while
c  lenstr reduces to the length of new string.
c----------------------------------------------------------------------
      i = 1
10    continue
         if (i .gt. lenstr) return
         if (str(i:i) .ne. ' ') then
            i = i + 1
            go to 10
         end if
         lenstr = lenstr - 1
         j = i
20       continue
            str(j:j) = str(j+1:j+1)
            j = j + 1
         if (j .le. lenstr) go to 20
         str(j+1:j+1) = ' '
      go to 10
      end
